#pragma once

// Desired LCD framerate. Should be between 55 and 75.
#define LCD_FPS (55)

// Defines the plotting mode.
enum LCD_PLOT_MODE {
	LCD_PLOT_MODE_SET,    // Pixels are set.
	LCD_PLOT_MODE_CLEAR,  // Pixels are cleared.
	LCD_PLOT_MODE_INVERT, // Pixels are inverted.
};

// Stores a 320x240 monochrome bitmap.
extern unsigned char __attribute__((far)) lcd_frame_buffer[9600];

// Keeps track of the current row and column when retracing the LCD.
extern volatile unsigned int lcd_scan_row;
extern volatile unsigned int lcd_scan_column;

// Initialises the LCD.
void lcd_init(void);

// Clear the LCD.
void lcd_clear(void);

// Set the current plotting mode.
void lcd_set_plot_mode(enum LCD_PLOT_MODE mode);

// Set a pixel on the LCD.
void lcd_set_pixel(int x, int y);

// Clear a pixel on the LCD.
void lcd_clear_pixel(int x, int y);

// Invert a pixel on the LCD.
void lcd_invert_pixel(int x, int y);

// Draw a pixel on the LCD using the current plotting mode.
extern void (*lcd_draw_pixel)(int, int);

// Draw a line on the LCD.
// Adapted from http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
void lcd_draw_line(int x0, int y0, int x1, int y1);

// Draw a circle on the LCD.
// Adapted from http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
void lcd_draw_circle(int x0, int y0, int radius);
