; Latches used for outputting data and control signals.
.equ LCD_DATA_LATCH, LATA
.equ LCD_CONTROL_LATCH, LATB

; Bit indices within the control latch corresponding to LCD control pins.
.equ LCD_FLM, 15
.equ LCD_M, 14
.equ LCD_C1, 13
.equ LCD_C2, 12
.equ LCD_D_OFF, 11

.text

; This interrupt service routine is used to retrace the LCD, outputting one 
.global __T1Interrupt
__T1Interrupt:

	; Preserve registers used by this function on the stack.
	push w0
	push w1

	bset LCD_CONTROL_LATCH, #LCD_D_OFF

	; Are we starting a new row?
	cp0 _lcd_scan_column
	bra nz, not_starting_new_row

		; Are we starting a new frame?
		cp0 _lcd_scan_row
		bra nz, not_starting_new_frame			
			; Toggle the M pin to provide the LCD AC voltage.
			btg LCD_CONTROL_LATCH, #LCD_M
			; Reset the scanning pointer.
			mov #edsoffset(_lcd_frame_buffer), w0
			mov w0, _lcd_scan_ptr
		not_starting_new_frame:

		; Begin the line.
		bset LCD_CONTROL_LATCH, #LCD_C1
		nop
		bclr LCD_CONTROL_LATCH, #LCD_C1

		; Clear the frame start line...
		bclr LCD_CONTROL_LATCH, #LCD_FLM
		bra nz, keep_flm_low
			; ...unless we are actually starting a new frame.
			bset LCD_CONTROL_LATCH, #LCD_FLM
		keep_flm_low:

	not_starting_new_row:

	; Output a column.

	mov _lcd_scan_ptr, w0

	btsc _lcd_scan_column, #0
	bra _output_right_column

	_output_left_column:
	
		; Output the four pixels from the "left" of the byte.
		mov.b [w0], w1
		swap.b w1
		bra _output_column_data
	
	_output_right_column:

		; Output the four pixels from the "right" of the byte.
		mov.b [w0++], w1
		mov w0, _lcd_scan_ptr

_output_column_data:
	; Output the data and issue the clock signal.
	bset LCD_CONTROL_LATCH, #LCD_C2
	mov w1, LCD_DATA_LATCH
	bclr LCD_CONTROL_LATCH, #LCD_C2

_increment_column_counter:
	; Increment the column counter and check if we've hit the right edge.
	inc _lcd_scan_column
	mov #80, w0
	cp _lcd_scan_column
	bra nz, not_finished_row
		
		; Clear the column counter.
		clr _lcd_scan_column

		; Increment the row counter.
		inc _lcd_scan_row
	
		; Have we reached line 240?
		mov #240, w0
		cp _lcd_scan_row
		bra nz, not_finished_frame
			; Yes, so reset the row counter.
			clr _lcd_scan_row
		not_finished_frame:
	
	not_finished_row:

	; Restore registers used by this function from the stack.
	pop w1
	pop w0

	; Clear the interrupt flag.
	bclr IFS0, #3 ; Symbol?
	retfie

.end
